require 'helper'

describe Resolver::DataMapper do

  subject { Resolver::DataMapper.new(mapping) }

  # entites
  let(:foo) { faked_class('Foo').new }
  let(:bar) { faked_class('Nested::Bar').new }

  # data mapper
  FooDataMapper = Class.new(Store::DataMapper)
  BarDataMapper = Class.new(Store::DataMapper)

  let(:mapping) do
    {
    'Foo' => FooDataMapper,
    'Bar' => BarDataMapper,
    }
  end

  it 'validates the subject' do
    assert_instance_of Resolver::DataMapper, subject
  end

  it 'resolves data mapper class from Store::Ref' do
    ref = Store::Ref.new('Foo', 42)
    assert_equal FooDataMapper, subject.resolve_data_mapper(ref)
  end

  it 'resolves data mapper class from Store::Query' do
    query = Store::Query.new('Foo', :query_name, :some, :args)
    assert_equal FooDataMapper, subject.resolve_data_mapper(query)
  end

  it 'resolves data mapper class from object' do
    assert_equal FooDataMapper, subject.resolve_data_mapper(foo)
  end

  it 'resolves data mapper class from namespaced object' do
    assert_equal BarDataMapper, subject.resolve_data_mapper(bar)
  end

  it 'includes Resolver module' do
    assert subject.respond_to? :extract_entity_class_name_from_ref
    assert subject.respond_to? :extract_entity_class_name_from_query
    assert subject.respond_to? :extract_entity_class_name_from_object
  end

end
