require 'helper'

describe Resolver::EntityMapper do
  subject { Resolver::EntityMapper.new(store, mapping) }

  # entites
  let(:foo) { faked_class('Foo').new }
  let(:bar) { faked_class('Nested::Bar').new }

  # entity mapper
  FooEntityMapper = Class.new(Store::EntityMapper)
  BarEntityMapper = Class.new(Store::EntityMapper)

  let(:mapping) do
    {
    'Foo' => FooEntityMapper,
    'Bar' => BarEntityMapper,
    }
  end

  let(:store) { Object.new }

  it 'validates the subject' do
    assert_instance_of Resolver::EntityMapper, subject
  end

  it 'resolves entity mapper instance from Store::Ref' do
    ref = Store::Ref.new('Foo', 42)
    assert_instance_of FooEntityMapper, subject.resolve_entity_mapper(ref)
  end

  it 'resolves entity mapper instance from Store::Query' do
    query = Store::Query.new('Foo', :query_name, :some, :args)
    assert_instance_of FooEntityMapper, subject.resolve_entity_mapper(query)
  end

  it 'resolves entity mapper instance from object' do
    assert_instance_of FooEntityMapper, subject.resolve_entity_mapper(foo)
  end

  it 'resolves entity mapper instance from namespaced object' do
    assert_instance_of BarEntityMapper, subject.resolve_entity_mapper(bar)
  end

  it 'includes Resolver module' do
    assert subject.respond_to? :extract_entity_class_name_from_ref
    assert subject.respond_to? :extract_entity_class_name_from_query
    assert subject.respond_to? :extract_entity_class_name_from_object
  end

end
