require 'helper'

describe 'Store::Repo' do
  let(:repo) do 
    repo = Store::Repo.new(:a, :b, :c) 
    repo.instance_variable_set :@store, store
    repo
  end

  let(:store) { MiniTest::Mock.new }

  it 'accepts three args' do
    Store::Repo.new(:a, :b, :c)
  end

  it 'delegeates save to @store and swallows result' do
    store.expect :save, :result, [:arg]

    result = repo.save(:arg)

    assert store.verify
    assert_equal true, result
  end

  it 'delegates remove to @store and swallows result' do
    store.expect :remove, :result, [:arg]
    
    result =  repo.remove(:arg)

    assert store.verify
    assert_equal true, result
  end

  it 'delegates build to @store and returns result without args' do
    store.expect :build, :result, [:arg]

    result = repo.build(:arg)

    assert store.verify
    assert_equal :result, result
  end

  it 'delegates build to @store and returns result with args' do
    store.expect :build, :result, [:arg, :a, :b]

    result = repo.build(:arg, :a, :b)

    assert store.verify
    assert_equal :result, result
  end

  it 'delegates save_and_load to @store' do
    store.expect :save, :refs, [:arg]
    store.expect :load, :entities, [:refs]

    result = repo.save_and_load(:arg)

    assert store.verify
    assert_equal :entities, result 
  end

  it 'delegeates reload to @store' do
    store.expect :reload, :result, [:arg]

    result =  repo.reload(:arg)

    assert store.verify
    assert_equal :result, result
  end

  describe 'private' do
    it 'delegeates query to @store' do
      store.expect :query, :result, [:arg]

      result =  repo.send :query, :arg

      assert store.verify
      assert_equal :result, result
    end

    it 'delegeates reference to @store' do
      store.expect :reference, :result, [:arg]

      result =  repo.send :reference, :arg

      assert store.verify
      assert_equal :result, result
    end
  end

end
